@@ -1,6 +1,6 @@
 package Git::Repository::Plugin::GitHooks;
 # ABSTRACT: A Git::Repository plugin with some goodies for hook developers
-$Git::Repository::Plugin::GitHooks::VERSION = '2.1.8';
+$Git::Repository::Plugin::GitHooks::VERSION = '2.2.0';
 use parent qw/Git::Repository::Plugin/;
 
 use 5.010;
@@ -31,7 +31,7 @@ sub _keywords {                 ## no critic (ProhibitUnusedPrivateSubroutines)
 
           filter_files_in_index filter_files_in_range filter_files_in_commit
 
-          authenticated_user
+          authenticated_user repository_name
 
           get_current_branch get_sha1 get_head_or_empty_tree
 
@@ -152,6 +152,8 @@ sub _prepare_gerrit_args {
 
     @$args = (\%opt);
 
+    $git->{_plugin_githooks}{gerrit_args} = \%opt;
+
     return;
 }
 
@@ -674,11 +676,17 @@ sub get_commit {
 }
 
 sub get_commits {
-    my ($git, $old_commit, $new_commit) = @_;
+    my ($git, $old_commit, $new_commit, $options, $paths) = @_;
 
     my $cache = $git->cache('ranges');
 
-    my $range = "$old_commit:$new_commit";
+    my $range = join(
+        ':',
+        $old_commit,
+        $new_commit,
+        defined $options ? join('', @$options) : '',
+        defined $paths   ? join('', @$paths)   : '',
+    );
 
     unless (exists $cache->{$range}) {
         # We're interested in all commits reachable from $new_commit but
@@ -732,7 +740,13 @@ sub get_commits {
 
         push @excludes, "^$old_commit" unless $old_commit eq $git->undef_commit;
 
-        $cache->{$range} = [$git->log($new_commit, @excludes)];
+        my @arguments;
+
+        push @arguments, @$options if defined $options;
+        push @arguments, $new_commit, @excludes;
+        push @arguments, '--', @$paths if defined $paths;
+
+        $cache->{$range} = [$git->log(@arguments)];
     }
 
     return @{$cache->{$range}};
@@ -826,18 +840,9 @@ sub get_affected_ref_range {
 }
 
 sub get_affected_ref_commits {
-    my ($git, $ref) = @_;
+    my ($git, $ref, $options, $paths) = @_;
 
-    my $affected = _get_affected_refs_hash($git);
-
-    exists $affected->{$ref}
-        or croak __PACKAGE__, ": get_affected_ref_commits($ref): no such affected ref\n";
-
-    unless (exists $affected->{$ref}{commits}) {
-        $affected->{$ref}{commits} = [$git->get_commits($git->get_affected_ref_range($ref))];
-    }
-
-    return @{$affected->{$ref}{commits}};
+    return $git->get_commits($git->get_affected_ref_range($ref), $options, $paths);
 }
 
 sub filter_files_in_index {
@@ -850,9 +855,9 @@ sub filter_files_in_index {
 }
 
 sub filter_files_in_range {
-    my ($git, $filter, $from, $to) = @_;
+    my ($git, $filter, $from, $to, $options, $paths) = @_;
 
-    # If $to is he undefined commit this means that a branch or tag is being
+    # If $to is the undefined commit this means that a branch or tag is being
     # removed. In this situation we return the empty list, bacause no file
     # has been affected.
     return if $to eq $git->undef_commit;
@@ -864,7 +869,7 @@ sub filter_files_in_range {
         # the files changed from the list's first commit's PARENT commit to
         # the list's last commit.
 
-        if (my @commits = $git->get_commits($from, $to)) {
+        if (my @commits = $git->get_commits($from, $to, $options, $paths)) {
             if (my @parents = $commits[0]->parent()) {
                 $from = $parents[0];
             } else {
@@ -929,6 +934,30 @@ sub authenticated_user {
     return $git->{_plugin_githooks}{authenticated_user};
 }
 
+sub repository_name {
+    my ($git) = @_;
+
+    unless (exists $git->{_plugin_githooks}{repository_name}) {
+        if (my $gerrit_args = $git->{_plugin_githooks}{gerrit_args}) {
+            # Gerrit
+             $git->{_plugin_githooks}{repository_name} = $gerrit_args->{'--project'};
+        } elsif (exists $ENV{STASH_REPO_NAME}) {
+            # Bitbucket
+            $git->{_plugin_githooks}{repository_name} = "$ENV{STASH_PROJECT_KEY}/$ENV{STASH_REPO_NAME}";
+        } else {
+            # As a last resort, return GIT_DIR's basename
+            my $gitdir = path($git->git_dir());
+            my $basename = $gitdir->basename;
+            if ($basename eq '.git') {
+                $basename = $gitdir->parent->basename;
+            }
+            $git->{_plugin_githooks}{repository_name} = $basename;
+        }
+    }
+
+    return $git->{_plugin_githooks}{repository_name};
+}
+
 sub get_current_branch {
     my ($git) = @_;
     my $branch = $git->run({fatal => [-129, -128]}, qw/symbolic-ref HEAD/);
@@ -1132,7 +1161,7 @@ Git::Repository::Plugin::GitHooks - A Git::Repository plugin with some goodies f
 
 =head1 VERSION
 
-version 2.1.8
+version 2.2.0
 
 =head1 SYNOPSIS
 
@@ -1362,7 +1391,7 @@ Git|https://stackoverflow.com/questions/9765453/is-gits-semi-secret-empty-tree-o
 
 Returns a L<Git::Repository::Log> object representing COMMIT.
 
-=head2 get_commits OLDCOMMIT NEWCOMMIT
+=head2 get_commits OLDCOMMIT NEWCOMMIT [OPTIONS [PATHS]]
 
 Returns a list of L<Git::Repository::Log> objects representing every commit
 reachable from NEWCOMMIT but not from OLDCOMMIT.
@@ -1382,6 +1411,24 @@ branch name prefixed by carets. We can get at their names using the
 technique described in, e.g., L<this
 discussion|http://stackoverflow.com/questions/3511057/git-receive-update-hooks-and-new-branches>.
 
+The L<Git::Repository::Log> objects are constructed ultimately by invoking the
+C<git log> command like this:
+
+  git log [<options>] <revision range> [-- <paths>]
+
+The C<revision range> is usually just C<OLDCOMMIT..NEWCOMMIT>, but there are
+some special cases which require some calculating as discussed above.
+
+The C<OPTIONS> optional argument is an array-ref pointing to an array of
+strings, which will be passed as options to the git-log command. It may be
+useful to grok some extra information about each commit (e.g., using
+C<--name-status>).
+
+The C<PATHS> optional argument is an array-ref pointing to an array of strings,
+which will be passed as pathspecs to the git-log command. It may be useful to
+filter the list of commits, grokking only those affecting specific paths in the
+repository.
+
 =head2 read_commit_msg_file FILENAME
 
 Returns the relevant contents of the commit message file called
@@ -1440,12 +1487,15 @@ command. It's useful in the C<update> and the C<pre-receive> hooks.
 Returns the two-element list of commit ids representing the OLDCOMMIT and
 the NEWCOMMIT of the affected REF.
 
-=head2 get_affected_ref_commits REF
+=head2 get_affected_ref_commits REF [OPTIONS [PATHS]]
 
 Returns the list of commits leading from the affected REF's NEWCOMMIT to
 OLDCOMMIT. The commits are represented by L<Git::Repository::Log> objects,
 as returned by the C<get_commits> method.
 
+The optional arguments OPTIONS and PATHS are passed to the C<get_commits>
+method.
+
 =head2 filter_files_in_index FILTER
 
 Returns a list of the names of the files that are changed in the index
@@ -1468,7 +1518,7 @@ is documented like this:
     other criteria in the comparison; if there is no file that matches other
     criteria, nothing is selected.
 
-=head2 filter_files_in_range FILTER, FROM, TO
+=head2 filter_files_in_range FILTER FROM TO [OPTIONS [PATHS]]
 
 Returns a list of the names of the files that are changed between commits
 FROM and TO. It's useful in the C<update> and the C<pre-receive> hooks when
@@ -1490,6 +1540,9 @@ calculate the difference between the first commit's parent and TO. When the
 first commit has no parent (in case it's a root commit) we return an empty
 list.
 
+The optional arguments OPTIONS and PATHS are passed to the C<get_commits>
+method.
+
 =head2 filter_files_in_commit FILTER, COMMIT
 
 Returns a list of the names of the files that are changed in COMMIT. It's
@@ -1517,6 +1570,12 @@ groks it from the C<githooks.userenv> configuration variable specification,
 which is described in the L<Git::Hooks> documentation. It's useful for most
 access control check plugins.
 
+=head2 repository_name
+
+Returns the repository name as a string. Currently it knows how to grok the name
+from Gerrit and Bitbucket servers. Otherwise it tries to grok it from the
+C<GIT_DIR> environment variable, which holds the path to the Git repository.
+
 =head2 get_current_branch
 
 Returns the repository's current branch name, as indicated by the C<git
